#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libps.h>

#include "lib2d.h"
#include "pad.h"
#include "addrs.h"

typedef enum
{
	MISSED,
	P1_HIT_P2,
	P2_HIT_P1
} Hit;

void main();
void RenderWorld();
void ProcessUserInput( unsigned short wPad, int iWho );
Hit CheckCollision( int iWho );                                    
int RectOverlap( RECT* r0, RECT* r1 );
void SetPlayerFrame( int iWho );

GsSPRITE g_PlayerSprite[2];
#define PS_SIZE (32)
GsSPRITE g_BonusSprite[1];
GsSPRITE g_PointerSprite;
GsSPRITE g_GaugeSprite;

int g_iMove[2];
int g_iScore[2];
int g_iAnimCount[2];
int g_iOriginU[2];
int g_iBonusShow;	

void main()
{
short angle = 4*4096;	/* rotate 4 degrees each frame */
unsigned long ulPad;	/* controller status */
Hit eHit = MISSED;		/* Collision flag */
GsSPRITE TitleSprite;	/* Big title sprite */
long lBonus;			/* Holds random number for bonus position */
int i;					/* Temp counter */

	// Standard Initialisation calls.
	PadInit();
	InitGraphics();
	
	// This example game initialisation.
	FntLoad( 960, 256 );
	FntOpen( 16, 180, 256, 24, 0, 64 );
	SetSpriteInfo( &TitleSprite, LEECHES_TIM, SCREEN_WIDTH>>1, SCREEN_HEIGHT>>2 );
	SetSpriteInfo( &g_PointerSprite, POINTER_TIM, SCREEN_WIDTH>>1, 20 );
	SetSpriteInfo( &g_GaugeSprite, GAUGE_TIM, SCREEN_WIDTH>>1, 20 );
	SetSpriteInfo( &g_BonusSprite[0], BONUS_TIM, 0, 0 );
	SetSpriteInfo( &g_PlayerSprite[0], ANILEECH_TIM, 107, 120 );
	g_PlayerSprite[0].w = PS_SIZE;
	g_PlayerSprite[0].h = PS_SIZE;
	g_PlayerSprite[0].mx = (PS_SIZE>>1);
	g_iOriginU[0] = g_PlayerSprite[0].u;
	
	SetSpriteInfo( &g_PlayerSprite[1], ELPMAXE_TIM, 214, 120 );

	// Main loop.
	while(1)
	{
		// Title.
		do
		{
			ulPad = PadRead();
			if(ulPad == (PADselect | PADstart)) goto Quit;
			RenderPrepare();
			FntPrint( "\tPlayer 1: %d\t%d :Player 2\n\n", g_iScore[0], g_iScore[1] );
			FntPrint( "\tPlease press START to play" );
			FntFlush( -1 );
			DrawSprite( &TitleSprite, 0 );
			RenderFinish();
		}while(ulPad != PADstart);
		
		// Initialise game parameters.
		g_PointerSprite.x = SCREEN_WIDTH>>1;
		g_iBonusShow = 0;
		g_PlayerSprite[0].x = SCREEN_WIDTH>>2;
		g_PlayerSprite[1].x = (SCREEN_WIDTH*3)>>2;
		for( i=0; i<2; i++ )
		{
			g_PlayerSprite[i].y = SCREEN_HEIGHT>>1;
			g_iMove[i]=1;
			g_iAnimCount[i]=0;
		}
		
		// Game loop (ends when pointer reaches an end of the power bar).
		do
		{
			ulPad = PadRead();
			// Animate red player.
			SetPlayerFrame( 0 );
			ProcessUserInput(ulPad & 0xFFFF, 0);
			if( P2_HIT_P1 != eHit) eHit = CheckCollision(0);
			// Animate ylw player.
			g_PlayerSprite[1].rotate += angle;
			ProcessUserInput((ulPad & 0xFFFF0000)>>16, 1);
			if( P1_HIT_P2 != eHit) eHit = CheckCollision(1);
		
			// Switch on a bonus?
			if(!g_iBonusShow)
			{
			 	if( rand()<32 ) // Approx 1 frame in 1000.
				{
					// Set the flag and put somewhere on the screen.
					// (could be problem if positioned too close to edge)
					g_iBonusShow++;
					lBonus = rand();
					g_BonusSprite[0].x = lBonus % SCREEN_WIDTH;
					g_BonusSprite[0].y = lBonus % SCREEN_HEIGHT;
				}
			}
			
			// Output to screen.
			RenderWorld();
		}while(	(g_PointerSprite.x > ((SCREEN_WIDTH>>1) - 128))
			  &&	(g_PointerSprite.x < ((SCREEN_WIDTH>>1) + 128)) );
			  
		// Update game score.
		if(g_PointerSprite.x > ((SCREEN_WIDTH>>1) - 128))
			g_iScore[1]++;
		else
			g_iScore[0]++;
	};
	
Quit:

	// clean up
	ResetGraph(3);
}

void RenderWorld()
{
	RenderPrepare();
	DrawSprite( &g_PlayerSprite[0], 10);
	DrawSprite( &g_PlayerSprite[1], 11);
	DrawSprite( &g_GaugeSprite, 20);
	DrawSprite( &g_PointerSprite, 19);
	if(g_iBonusShow)
		DrawSprite( &g_BonusSprite[0], 40 );
	
	RenderFinish();
}

void ProcessUserInput( unsigned short wPad, int iWho )
{
	if((wPad & PADLleft) && (g_PlayerSprite[iWho].x > (g_PlayerSprite[iWho].w>>1)))
	{
		g_PlayerSprite[iWho].x -= g_iMove[iWho];
		g_PlayerSprite[iWho].scalex = 1<<12;
		g_PlayerSprite[iWho].rotate = 0;
	}
	if((wPad & PADLright) && (g_PlayerSprite[iWho].x < (SCREEN_WIDTH - (g_PlayerSprite[iWho].w>>1))))
	{
		g_PlayerSprite[iWho].x += g_iMove[iWho];
		g_PlayerSprite[iWho].scalex = -1<<12;
		g_PlayerSprite[iWho].rotate = 0;
	}
	if((wPad & PADLup) && (g_PlayerSprite[iWho].y > (g_PlayerSprite[iWho].h>>1)))
	{
		g_PlayerSprite[iWho].y -= g_iMove[iWho];
		g_PlayerSprite[iWho].rotate = 90<<12;
	}
	if((wPad & PADLdown) && (g_PlayerSprite[iWho].y < (SCREEN_HEIGHT - (g_PlayerSprite[iWho].h>>1))))
	{
		g_PlayerSprite[iWho].y += g_iMove[iWho];
		g_PlayerSprite[iWho].rotate = -90<<12;
	}
}

Hit CheckCollision( int iWho )
{
	RECT r[2];

	// Get Player collision sensitive rectangles.
	r[0].x = g_PlayerSprite[0].x;
	r[0].w = g_PlayerSprite[0].w>>1;
	r[0].y = g_PlayerSprite[0].y;
	r[0].h = g_PlayerSprite[0].h>>1;
	r[1].x = g_PlayerSprite[1].x;
	r[1].w = g_PlayerSprite[1].w>>1;
	r[1].y = g_PlayerSprite[1].y;
	r[1].h = g_PlayerSprite[1].h>>1;
	
	// Check for collision with bonuses.
	if( g_iBonusShow )
	{
		// Use an evil hack as GsSPRITE has xyhw elements in the same order as RECT.
		if( RectOverlap( &r[iWho], (RECT*)&(g_BonusSprite[0].x) ))
		{
			if(0 == iWho)
				g_PointerSprite.x -= 20;	// Player 0 (red) gets bonus.
			else
				g_PointerSprite.x += 20;	// Player 1 (ylw) gets bonus.
			g_iBonusShow = 0;
		}
	}	

	// Check for player collision.
	if( RectOverlap( &r[0], &r[1] ))
		// Collided.
	{
		g_iMove[iWho] = -1;	/* Bounce off by reversing controls */
		
		if( 0 == iWho )
		{
			g_PointerSprite.x--;
			return P1_HIT_P2;
		}
		else
		{
			g_PointerSprite.x++;
			return P2_HIT_P1;
		}
	}
	else
	{
		g_iMove[iWho] = 1;	// Back to normal controls.
		return MISSED;
	}
}

/* 	Checks if two rectangular areas overlap
	Returns zero if no overlap condition */
int RectOverlap( RECT* r0, RECT*r1 )
{
	return( 	((r0->x + r0->w) >= r1->x )
		&&   ((r0->y + r0->h) >= r1->y )
		&&	(r0->x <= (r1->x + r1->w))
		&&	(r0->y <= (r1->y + r1->h)) );
}

void SetPlayerFrame( int iWho )
{
	// Move to next frame (assumes split into 8x1 grid, bit stuff slows down switch).
	g_PlayerSprite[iWho].u = g_iOriginU[iWho] + (PS_SIZE * ((g_iAnimCount[iWho] & 0x70)>>4));
	g_iAnimCount[iWho]++;
}
